<?php
/**
 * The driver class.
 * 
 * @package webkdb
 */
if(!defined("WEBK_BASEDIR")) die("Cannot open this file! Must be included.");

require_once('query.class.php');
require_once('result.class.php');
require_once('qb.class.php');
require_once('exceptions.php');

/**
 * The driver for the MySQL database connection and controlling.
 *
 * @package webkdb
 * @author thg2oo6
 * @copyright Copyright (c) 2012
 * @version 1.0
 * @access public
 */
class WebKDBDriver{
	/**
	 * The mysql connection link.
	 *
	 * @var mysqli $_connection
	 */
	private $_connection;

	/**
	 *  The last executed query.
	 *
	 *	@var WQuery $_lastQ
	 */
	private $_lastQ = NULL;

	/**
	 * Initialises the MySQL driver.
	 *
	 * @param string $hostname The address of the mysql server.
	 * @param string $username The username used to connect to the database.
	 * @param string $password The password used to connect to the database.
	 * @param string $database The name of the database to be used.
	 */
	function __construct($hostname="localhost",$username="root",$password="",$database="webk_test"){
		if(empty($hostname)) $hostname="localhost";
		if(empty($username)) $username="root";
		/*The password can be empty. - $password*/
		if(empty($database)) $database="webk_test";

		$this->_connection = new mysqli($hostname,$username,$password,$database);

		if(mysqli_connect_error())
			throw new DBException('Connect Error ('.mysqli_connect_errno().') '.mysqli_connect_error());
	}

	/**
	 * Executes a query and returns its result.
	 *
	 * @param string $sql The query to be executed.
	 * @return WResult
	 */
	final public function query($sql){
		if(!is_null($this->_lastQ)) unset($this->_lastQ);
		$this->_lastQ = new WQuery($this->_connection);

		$q=$this->_lastQ->query($sql);

		return new WResult($q);
	}

	/**
	 * Returns the string of the last executed query.
	 *
	 * @return string
	 */
	final public function last_query(){
		return $this->_lastQ->last_query();
	}

	/**
	 * Returns the ID of the last executed insert query.
	 *
	 * @return integer
	 */
	final public function insert_id(){
		return $this->_lastQ->insert_id();
	}

	/**
	 * Tests if there were errors during the execution of the last query.
	 * If thee were errors, the function will throw an DBException.
	 *
	 * @return void
	 */
	final public function error(){
		if($this->_lastQ->error())
			throw new DBException($this->_lastQ->error_str());
	}

	/**
	 * Returns an array with the database results represented as objects.
	 *
	 * @return array
	 */
	final public function result(){
		$_result = new WResult($this->_lastQ->query());
		return $_result->result();
	}

	/**
	 * Returns an array with the database results represented as associative array.
	 *
	 * @return array
	 */
	final public function result_array(){
		$_result = new WResult($this->_lastQ->query());
		return $_result->result_array();
	}

	/**
	 * Returns a row from the select query (object). In case of multiple rows it returns
	 * only the first one.
	 *
	 * @return object
	 */
	final public function row(){
		$_result = new WResult($this->_lastQ->query());
		return $_result->row();
	}

	/**
	 * Returns a row from the select query (array). In case of multiple rows it returns
	 * only the first one.
	 *
	 * @return array
	 */
	final public function row_array(){
		$_result = new WResult($this->_lastQ->query());
		return $_result->row_array();
	}

	/**
	 * Returns the number of the rows returned by the executed query.
	 *
	 * @return integer
	 */
	final public function num_rows(){
		$_result = new WResult($this->_lastQ->query());
		return $_result->num_rows();
	}

	/**
	 * Generates a SELECT query string.
	 *
	 * @param string $table The table to execute the SELECT query.
	 * @return QueryBuilder
	 */
	final public function select($table){
		return QueryBuilder::driver_exec($this,"SELECT",$table);
	}

	/**
	 * Generates a INSERT query string.
	 *
	 * @param string $table The table to execute the INSERT query.
	 * @return QueryBuilder
	 */
	final public function insert($table){
		return QueryBuilder::driver_exec($this,"INSERT",$table);
	}

	/**
	 * Generates a UPDATE query string.
	 *
	 * @param string $table The table to execute the UPDATE query.
	 * @return QueryBuilder
	 */
	final public function update($table){
		return QueryBuilder::driver_exec($this,"UPDATE",$table);
	}

	/**
	 * Generates a DELETE query string.
	 *
	 * @param string $table The table to execute the DELETE query.
	 * @return QueryBuilder
	 */
	final public function delete($table){
		return QueryBuilder::driver_exec($this,"DELETE",$table);
	}

	/**
	 * Generates a TRUNCATE query string.
	 *
	 * @param string $table The table to execute the TRUNCATE query.
	 * @return QueryBuilder
	 */
	final public function truncate($table){
		return QueryBuilder::driver_exec($this,"TRUNCATE",$table);
	}

}

/*File: driver.php*/
/*Date: 16.07.2012*/
