<?php
/**
 * The query builder class used by the WebKDBDriver.
 *
 * @package webkdb
 */
if(!defined("WEBK_BASEDIR")) die("Cannot open this file! Must be included.");
require_once('exceptions.php');

/**
 * Helper for creating a mysql query (SELECT, INSERT, UPDATE, DELETE, TRUNCATE).
 *
 * @package webkdb
 * @author thg2oo6
 * @copyright Copyright (c) 2012
 * @version 1.0
 * @access public
 */
class QueryBuilder{
	/**
	 * Used for ORDER clause. Defines an ASCENDING order.
	 */
	const ASC="ASC";
	/**
	 * Used for ORDER clause. Defines an DESCENDING order.
	 */
	const DESC="DESC";

	/**
	 * The type of the Query string.
	 *
	 * @var string $_type
	 */
	private $_type;
	/**
	 * The table for the Query string.
	 *
	 * @var string $_table
	 */
	private $_table;
	/**
	 * The fields of the Query string.
	 *
	 * @var array $_fields
	 */
	private $_fields;
	/**
	 * The elements of the WHERE clause.
	 *
	 * @var array $_where
	 */
	private $_where=array();
	/**
	 * The elements of the ORDER clause.
	 *
	 * @var array $_order
	 */
	private $_order=array();
	/**
	 * The limit size of the Query string.
	 *
	 * @var integer $_limit_size
	 */
	private $_limit_size;
	/**
	 * From where the limit will start.
	 *
	 * @var integer $_limit_from
	 */
	private $_limit_from;
	/**
	 * If the Query is an INSERT or UPDATE it contains keys+values.
	 *
	 * @var array $_set
	 */
	private $_set=array();

	/**
	 * The connection with the database driver.
	 *
	 * @var WebKDBDriver $_dbcon
	 */
	public $_dbcon=NULL;

	/**
	 * Used to see if the query will be executed or not.
	 *
	 * @var boolean $_execute
	 */
	public $_execute=false;

	/**
	 * Constructs the QueryBuilder element.
	 *
	 * @param string $type The type of the SQL Query string.
	 * @param string $table The table where we perform the query.
	 * @param mixed $fields The fields requested.
	 */
	function __construct($type,$table,$fields="*"){
		if(empty($type)) $type="select";
		if(!is_string($type)) throw new QBException("Invalid TYPE.");
		$type=strtoupper($type);

		if($fields!="*"&&!empty($fields)) $this->_fields=$this->_fieldParser($fields);
		else $this->_fields=$fields;

		$this->_type=$type;
		$this->_table=$table;
	}

	/**
	 * A static initialisation for the QueryBuilder class. To be used only with
	 * the driver class.
	 *
	 * @param WebKDBDriver $db_con The connection with the database.
	 * @param string $type The type of the SQL Query string.
	 * @param string $table The table where we perform the query.
	 * @param mixed $fields The fields requested.
	 * @return QueryBuilder
	 */
	final static public function driver_exec(&$db_con,$type,$table,$fields="*"){
		$qb = new QueryBuilder($type,$table,$fields);
		$qb->_dbcon=$db_con;
		$qb->_execute=true;

		return $qb;
	}

	/**
	 * Executes the query.
	 *
	 * @return mixed
	 */
	final public function execute(){
		if(($this->_type=="INSERT" || $this->_type=="UPDATE") && empty($this->_set)) throw new QBException("What elements do you want to set?");

		$_str = $this->_qbtext();

		if(is_array($this->_fields))		$_fields = $this->_qbGlue($this->_fields,", ");
		elseif(is_string($this->_fields))	$_fields = $this->_fields;

		$_replace = array(
			"%fields%"	=>$_fields,
			"%table%"	=>"`{$this->_table}`",
			"%values%"	=>$this->_qbGlue($this->_set,", ")
		);
		$_str = str_replace(array_keys($_replace),array_values($_replace),$_str);

		$_str .= $this->_whereParse();
		$_str .= $this->_orderParse();
		$_str .= $this->_limitParse();

		//var_dump($_str);
		if($this->_execute) return $this->_dbcon->query($_str);
		else 				return $_str;
	}

	/**
	 * Adds a value to the SET clause used in UPDATE and INSERT queries.
	 *
	 * @param string $key The key to recive a value.
	 * @param string $value The value to be set for a key.
	 * @param string $type The type of the value (MYSQL, INTEGER, BLOB, STRING).
	 * @return void
	 */
	final public function set($key,$value,$type="string"){
		if(empty($key) || !is_string($key)) throw new QBException("Invalid Key sent!");
		if(empty($value) || !is_string($value)) throw new QBException("Invalid Value sent!");

		$type=strtolower($type);

		switch($type){
			case 'mysql':
			case 'integer':
			case 'blob':
				$this->_set[]="`{$key}`={$value}";
				break;

			case 'string':
			default:
				$value=addslashes($value);
				$this->_set[]="`{$key}`='{$value}'";
				break;
		}

	}

	/**
	 * Adds a WHERE clause to the SQL Query string.
	 *
	 * @param string $key The key needed in the query string.
	 * @param string $value The value for the key.
	 * @param string $operator The operator between the key and the value.
	 * @param string $linking The link between the elements of the WHERE clause.
	 * @return void
	 */
	final public function where($key,$value,$operator="=",$linking="AND"){
		if(empty($key) || !is_string($key)) throw new QBException("Invalid Key sent!");
		if(empty($value) || !is_string($value)) throw new QBException("Invalid Value sent!");
		if(empty($operator) || !is_string($operator)) $operator="=";
		if(empty($linking) || !is_string($linking)) $linking="AND";

		$_s="";
		if(count($this->_where)!=0)
			$_s="{$linking} ";

		$operator=strtoupper($operator);
		if(!in_array($operator,array("IN","NOT IN"))) $value="'".addslashes($value)."'";
		$this->_where[]=$_s."`{$key}` {$operator} {$value}";
	}

	/**
	 * Adds an ORDER clause to the SQL Query string.
	 *
	 * @param string $key The key after what we order.
	 * @param string $how The method of orderding (ASC/DESC).
	 * @return void
	 */
	final public function order($key,$how="ASC"){
		if(empty($key) || !is_string($key)) throw new QBException("Invalid Key sent!");
		if($key=="RAND()")
			$this->_order[]="`{$key}`";
		else{
			if(empty($how) || !is_string($how)) throw new QBException("Invalid Ordering method sent!");
			$this->_order[]="`{$key}` {$how}";
		}
	}

	/**
	 * Sets the LIMIT clause size.
	 *
	 * @param integer $size The size of the LIMIT.
	 * @return void
	 */
	final public function limit_size($size){
		if(is_integer($size))$this->_limit_size=$size;
	}

	/**
	 * Sets where to start the Query.
	 *
	 * @param integer $from The position from where the query will start.
	 * @return void
	 */
	final public function limit_from($from){
		if(is_integer($from))$this->_limit_from=$from;
	}

	/**
	 * Adds values for the LIMIT clause.
	 *
	 * @param integer $size The size of the query.
	 * @param integer $from From where the query will start.
	 * @return void
	 */
	final public function limit($size,$from=0){
		$this->limit_size($size);
		$this->limit_from($from);
	}

	/**
	 * Parses a list with fields and returns an array with the fields parsed for
	 * a MySQL query.
	 *
	 * @param mixed $fields The list of fields. It may be separated by:
	 * ',', ' ', '|'. Also it can be an array.
	 * @param boolean $hasKeys If the fields are sent as an associative array,
	 * this field must be set to true.
	 * @return array
	 */
	final private function _fieldParser($fields,$hasKeys=false){
		$_r = array();

		if(is_string($fields)){
			$fields = str_replace(array(", "," ,"),",",$fields);
			$_f=explode("|",str_replace(array(" ",","),"|",$fields));

			foreach($_f as $e) $_r[] = "`{$e}`";
		}elseif(is_array($fields)){
			$_f = $fields;

			if($hasKeys)	foreach($_f as $k=>$e) $_r["`{$k}`"] = $e;
			else			foreach($_f as $e) $_r[] = "`{$e}`";
		}else{
			throw new QBException("Invalid type for field parsing");
		}

		return $_r;
	}

	/**
	 * Method used for creating a string from an array.
	 *
	 * @param array $array The array to be parsed.
	 * @param string $glue The glue used to transform the array into a string.
	 * @return array
	 */
	final private function _qbGlue($array,$glue=" "){
		return implode($glue,$array);
	}

	/**
	 * Returns a base string for different SQL queries supported by the driver.
	 * (SELECT, INSERT, UPDATE,
	 *
	 * @return string
	 */
	final private function _qbtext(){
		switch($this->_type){
			case 'SELECT':	return "SELECT %fields% FROM %table%";break;
			case 'INSERT':	return "INSERT INTO %table% SET %values%";break;
			case 'UPDATE':	return "UPDATE %table% SET %values%";break;
			case 'DELETE':	return "DELETE FROM %table%";break;
			case 'TRUNCATE':return "TRUNCATE TABLE %table%";break;
			default: throw new QBException("Invalid Query Type!"); break;
		}
	}

	/**
	 * Creates the WHERE clause.
	 *
	 * @return string
	 */
	final private function _whereParse(){
		return count($this->_where)>0?"\nWHERE ".$this->_qbGlue($this->_where," "):"";
	}

	/**
	 * Creates the ORDER clause.
	 *
	 * @return string
	 */
	final private function _orderParse(){
		return count($this->_order)>0?"\nORDER by ".$this->_qbGlue($this->_order," "):"";
	}

	/**
	 * Creates the LIMIT clause.
	 *
	 * @return string
	 */
	final private function _limitParse(){
		return $this->_limit_size>0?"\nLIMIT ".$this->_limitFromParse().$this->_limit_size:"";
	}

	/**
	 * Parses the from part of the LIMIT clause.
	 *
	 * @return string
	 */
	final private function _limitFromParse(){
		$types=array("INSERT","TRUNCATE");
		if(!in_array($this->_type,$types))
			return $this->_limit_from>=0?$this->_limit_from.",":"";
		return "";
	}
}

/*File: qb.class.php*/
/*Date: 17.07.2012*/
